import 'dart:convert';
import 'dart:io';

import 'request.dart';

class Response {
  final Request request;
  final int statusCode;
  final HttpHeaders headers;
  final int contentLength;
  final Stream<List<int>> body;

  Response({
    required this.statusCode,
    required this.headers,
    required this.body,
    required this.contentLength,
    required this.request,
  });

  String? _raw;

  Future<String> text() async {
    _raw ??= await body.transform(utf8.decoder).join();
    return _raw!;
  }

  Future<dynamic> json() async {
    _raw ??= await body.transform(utf8.decoder).join();
    return _raw!.isNotEmpty ? jsonDecode(_raw!) : (<String, dynamic>{});
  }
}

class ErrorResponse<T> {
  final int statusCode;
  final String? reason;
  final T? data;
  final Response? response;

  ErrorResponse(
    this.statusCode, {
    this.response,
    this.data,
    this.reason,
  });

  @override
  String toString() {
    return data != null ? "$data" : "$statusCode";
  }
}
